<div data-role='page'>
  <div data-role='header'>
    {{>home}}
    <h1>{{label}}{{#outbox_id}} (Unsynced){{/outbox_id}}</h1>
    <a data-icon="edit" data-iconpos="notext" data-transition="flip"
       {{#outbox_id}}
       href="{{rt}}/outbox/{{outbox_id}}/edit"
       {{/outbox_id}}
       {{^outbox_id}}
       href="{{rt}}/items/{{id}}/edit"
       {{/outbox_id}}
       >Edit</a>
  </div>
  <div role="main" class="ui-content">
    <p id="label">{{label}}</p>
    <p id="color"><span style="color:{{color}}">{{color_label}}</span></p>
    <p id="type">{{#type}}
      <a href="{{rt}}{{#outbox}}/outbox/{{outbox_id}}{{/outbox}}{{^outbox}}/itemtypes/{{id}}{{/outbox}}">
        {{label}}
      </a>
    {{/type}}</p>
    {{#parent}}
    <p id="parent">
      <a href="{{rt}}{{#outbox}}/outbox/{{outbox_id}}{{/outbox}}{{^outbox}}/items/{{id}}{{/outbox}}">
        {{label}}
      </a>
    </p>
    {{/parent}}
    {{#values}}
    <p class="value" id="value-{{attribute_id}}">{{#attribute}}{{label}}{{/attribute}}: {{value}}</p>
    {{/values}}
    <hr>
    {{{html}}}
    <hr>
    <div id="item-{{id}}-map" style="height:400px;"></div>
    {{>context_test}}
{{>footer}}
