<div data-role='page'>
  <div data-role='header'>
    {{>home}}
    <h1>{{label}}{{#outbox_id}} (Unsynced){{/outbox_id}}</h1>
    <a data-icon="edit" data-iconpos="notext" data-transition="flip"
       {{#outbox_id}}
       href="{{rt}}/outbox/{{outbox_id}}/edit"
       {{/outbox_id}}
       {{^outbox_id}}
       href="{{rt}}/itemtypes/{{id}}/edit"
       {{/outbox_id}}
       >Edit</a>
  </div>
  <div role="main" class="ui-content">
    <p id="label">{{label}}</p>
    <ul data-role="listview" data-inset="true">
      <li><a href="{{rt}}/itemtypes/{{id}}/items">Items</a></li>
    </ul>
    {{>context_test}}
{{>footer}}
